@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: audio_esd_out.c,v 1.35 2006/07/16 16:18:09 dsalt Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -523,7 +521,7 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen,
 
   esd_close(audio_fd);
 
-  this                     = (esd_driver_t *) xine_xmalloc (sizeof (esd_driver_t));
+  this                     = calloc(1, sizeof (esd_driver_t));
   if (!this)
     return NULL;
   this->xine               = class->xine;
@@ -585,7 +583,7 @@ static void *init_class (xine_t *xine, void *data) {
 
   esd_class_t        *this;
 
-  this = (esd_class_t *) xine_xmalloc (sizeof (esd_class_t));
+  this = calloc(1, sizeof (esd_class_t));
   if (!this)
     return NULL;
 
