@@ -186,7 +186,7 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen,
 
   lprintf ("open_plugin called\n");
 
-  this = (none_driver_t *) xine_xmalloc (sizeof (none_driver_t));
+  this = calloc(1, sizeof (none_driver_t));
   if (!this)
     return NULL;
 
@@ -236,7 +236,7 @@ static void *init_class (xine_t *xine, void *data) {
 
   lprintf ("init class\n");
 
-  this = (none_class_t *) xine_xmalloc (sizeof (none_class_t));
+  this = calloc(1, sizeof (none_class_t));
   if (!this)
     return NULL;
 
