@@ -332,7 +332,7 @@ static int demux_wv_get_optional_data(demux_plugin_t *const this_gen,
 static demux_plugin_t *open_plugin (demux_class_t *const class_gen,
 				    xine_stream_t *const stream,
 				    input_plugin_t *const input) {
-  demux_wv_t *const this = xine_xmalloc (sizeof (demux_wv_t));
+  demux_wv_t *const this = calloc(1, sizeof (demux_wv_t));
   this->stream = stream;
   this->input = input;
 
@@ -401,7 +401,7 @@ static void class_dispose (demux_class_t *const this_gen) {
 }
 
 void *demux_wv_init_plugin (xine_t *const xine, void *const data) {
-  demux_wv_class_t *const this = xine_xmalloc (sizeof (demux_wv_class_t));
+  demux_wv_class_t *const this = calloc(1, sizeof (demux_wv_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
