@@ -91,6 +91,7 @@ struct ff_video_decoder_s {
   int               pts_tag_stable_counter;
 #endif /* AVCODEC_HAS_REORDERED_OPAQUE */
   int               video_step;
+  int               reported_video_step;
 
   uint8_t           decoder_ok:1;
   uint8_t           decoder_init_mode:1;
@@ -224,6 +225,11 @@ static int get_buffer(AVCodecContext *context, AVFrame *av_frame){
 
   av_frame->type= FF_BUFFER_TYPE_USER;
 
+#ifdef AVCODEC_HAS_REORDERED_OPAQUE
+  /* take over pts for this frame to have it reordered */
+  av_frame->reordered_opaque = context->reordered_opaque;
+#endif
+
   xine_list_push_back(this->dr1_frames, av_frame);
 
   return 0;
@@ -588,7 +594,7 @@ static void ff_convert_frame(ff_video_decoder_t *this, vo_frame_t *img) {
       img->width,
       this->bih.biHeight);
 
-  } else if (this->context->pix_fmt == PIX_FMT_RGBA32) {
+  } else if (this->context->pix_fmt == PIX_FMT_RGB32) {
           
     int x, plane_ptr = 0;
     uint32_t *argb_pixels;
@@ -1278,7 +1284,15 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
       }
 
       /* use externally provided video_step or fall back to stream's time_base otherwise */
-      video_step_to_use = (this->video_step || !this->context->time_base.den) ? this->video_step : (int)(90000ll * this->context->time_base.num / this->context->time_base.den);
+      video_step_to_use = (this->video_step || !this->context->time_base.den)
+                        ? this->video_step
+                        : (int)(90000ll
+#if LIBAVCODEC_VERSION_INT >= 0x341400
+                                * this->context->ticks_per_frame
+#elif LIBAVCODEC_VERSION_INT >= 0x340000
+# warning Building without avcodec ticks_per_frame support; you should upgrade your libavcodec and recompile
+#endif
+                                * this->context->time_base.num / this->context->time_base.den);
 
       /* aspect ratio provided by ffmpeg, override previous setting */
       if ((this->aspect_ratio_prio < 2) &&
@@ -1304,7 +1318,7 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
 
 	  /* initialize the colorspace converter */
 	  if (!this->cs_convert_init) {
-	    if ((this->context->pix_fmt == PIX_FMT_RGBA32) ||
+	    if ((this->context->pix_fmt == PIX_FMT_RGB32) ||
 	        (this->context->pix_fmt == PIX_FMT_RGB565) ||
 	        (this->context->pix_fmt == PIX_FMT_RGB555) ||
 	        (this->context->pix_fmt == PIX_FMT_BGR24) ||
@@ -1381,7 +1395,10 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
           /* fallback to the VIDEO_PTS_MODE */
           video_step_to_use = 0;
         }
-        
+
+        if (video_step_to_use && video_step_to_use != this->reported_video_step)
+          _x_stream_info_set(this->stream, XINE_STREAM_INFO_FRAME_DURATION, (this->reported_video_step = video_step_to_use));
+
         if (this->av_frame->repeat_pict)
           img->duration = video_step_to_use * 3 / 2;
         else
@@ -1445,7 +1462,7 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 
   if (buf->decoder_flags & BUF_FLAG_FRAMERATE) {
     this->video_step = buf->decoder_info[0];
-    _x_stream_info_set(this->stream, XINE_STREAM_INFO_FRAME_DURATION, this->video_step);
+    _x_stream_info_set(this->stream, XINE_STREAM_INFO_FRAME_DURATION, (this->reported_video_step = this->video_step));
   }
 
   if (buf->decoder_flags & BUF_FLAG_PREVIEW) {
