@@ -187,7 +187,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   demux_cdda_t   *this;
 
-  this         = xine_xmalloc (sizeof (demux_cdda_t));
+  this         = calloc(1, sizeof(demux_cdda_t));
   this->stream = stream;
   this->input  = input;
 
@@ -250,7 +250,7 @@ static void class_dispose (demux_class_t *this_gen) {
 void *demux_cdda_init_plugin (xine_t *xine, void *data) {
   demux_cdda_class_t     *this;
 
-  this = xine_xmalloc (sizeof (demux_cdda_class_t));
+  this = calloc(1, sizeof(demux_cdda_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
