@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 /*
@@ -64,8 +64,6 @@
  *   - scan through all 768 palette bytes
  *     - if any bytes exceed 63, do not shift the bytes at all before
  *       transmitting them to the video decoder
- *
- * $Id: demux_idcin.c,v 1.55 2007/02/20 00:34:55 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -474,7 +472,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   demux_idcin_t  *this;
 
-  this         = xine_xmalloc (sizeof (demux_idcin_t));
+  this         = calloc(1, sizeof(demux_idcin_t));
   this->stream = stream;
   this->input  = input;
 
@@ -549,7 +547,7 @@ static void class_dispose (demux_class_t *this_gen) {
 void *demux_idcin_init_plugin (xine_t *xine, void *data) {
   demux_idcin_class_t     *this;
 
-  this         = xine_xmalloc (sizeof (demux_idcin_class_t));
+  this         = calloc(1, sizeof(demux_idcin_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
