@@ -15,9 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: demux_shn.c,v 1.2 2006/09/28 22:27:52 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -67,8 +65,8 @@ static int open_shn_file(demux_shn_t *this) {
   if (_x_demux_read_header(this->input, peak, 4) != 4)
       return 0;
 
-  if ((peak[0] != 'a') && (peak[1] != 'j') &&
-      (peak[2] != 'k') && (peak[3] != 'g')) {
+  if ((peak[0] != 'a') || (peak[1] != 'j') ||
+      (peak[2] != 'k') || (peak[3] != 'g')) {
     return 0;
   }
 
@@ -90,7 +88,7 @@ static int demux_shn_send_chunk(demux_plugin_t *this_gen) {
   buf->pts = 0;
 
   bytes_read = this->input->read(this->input, buf->content, buf->max_size);
-  if (bytes_read == 0) {
+  if (bytes_read <= 0) {
     buf->free_buffer(buf);
     this->status = DEMUX_FINISHED;
     return this->status;
@@ -184,7 +182,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   demux_shn_t    *this;
 
-  this         = xine_xmalloc (sizeof (demux_shn_t));
+  this         = calloc(1, sizeof(demux_shn_t));
   this->stream = stream;
   this->input  = input;
 
@@ -202,7 +200,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   switch (stream->content_detection_method) {
 
   case METHOD_BY_EXTENSION: {
-    char *extensions, *mrl;
+    const char *extensions, *mrl;
 
     mrl = input->get_mrl (input);
     extensions = class_gen->get_extensions (class_gen);
@@ -230,19 +228,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-static char *get_description (demux_class_t *this_gen) {
+static const char *get_description (demux_class_t *this_gen) {
   return "Shorten demux plugin";
 }
 
-static char *get_identifier (demux_class_t *this_gen) {
+static const char *get_identifier (demux_class_t *this_gen) {
   return "Shorten";
 }
 
-static char *get_extensions (demux_class_t *this_gen) {
+static const char *get_extensions (demux_class_t *this_gen) {
   return "shn";
 }
 
-static char *get_mimetypes (demux_class_t *this_gen) {
+static const char *get_mimetypes (demux_class_t *this_gen) {
   return NULL;
 }
 
@@ -255,7 +253,7 @@ static void class_dispose (demux_class_t *this_gen) {
 void *demux_shn_init_plugin (xine_t *xine, void *data) {
   demux_shn_class_t     *this;
 
-  this = xine_xmalloc (sizeof (demux_shn_class_t));
+  this = calloc(1, sizeof(demux_shn_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
