@@ -15,11 +15,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * This file contains plugin entries for several demuxers used in games
- *
- * $Id: group_audio.c,v 1.26 2007/03/03 02:06:09 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -71,9 +69,11 @@ static const demuxer_info_t demux_info_mpc = {
   1                        /* priority */
 };
 
+#ifdef HAVE_NOSEFART
 static const demuxer_info_t demux_info_nsf = {
   10                       /* priority */
 };
+#endif
 
 static const demuxer_info_t demux_info_realaudio = {
   10                       /* priority */
@@ -120,7 +120,9 @@ const plugin_info_t xine_plugin_info[] EXPORTED = {
   { PLUGIN_DEMUX, 26, "flac",      XINE_VERSION_CODE, &demux_info_flac,      demux_flac_init_plugin },
   { PLUGIN_DEMUX, 26, "mp3",       XINE_VERSION_CODE, &demux_info_mpgaudio,  demux_mpgaudio_init_class },
   { PLUGIN_DEMUX, 26, "mpc",       XINE_VERSION_CODE, &demux_info_mpc,       demux_mpc_init_plugin },
+#ifdef HAVE_NOSEFART
   { PLUGIN_DEMUX, 26, "nsf",       XINE_VERSION_CODE, &demux_info_nsf,       demux_nsf_init_plugin },
+#endif
   { PLUGIN_DEMUX, 26, "realaudio", XINE_VERSION_CODE, &demux_info_realaudio, demux_realaudio_init_plugin },
   { PLUGIN_DEMUX, 26, "shn",       XINE_VERSION_CODE, &demux_info_shn,       demux_shn_init_plugin },
   { PLUGIN_DEMUX, 26, "snd",       XINE_VERSION_CODE, &demux_info_snd,       demux_snd_init_plugin },
