@@ -241,7 +241,7 @@ static int id3v2_parse_header(input_plugin_t *input, uint8_t *mp3_frame_header,
 
     lprintf("tag: ID3 v2.%d.%d\n", mp3_frame_header[3], tag_header->revision);
     lprintf("flags: %d\n", tag_header->flags);
-    lprintf("size: %d\n", tag_header->size);
+    lprintf("size: %zu\n", tag_header->size);
     return 1;
   } else {
     return 0;
@@ -261,7 +261,7 @@ static int id3v22_parse_frame_header(input_plugin_t *input,
 
     frame_header->size = _X_BE_24_synchsafe(&buf[3]);
 
-    lprintf("frame: %c%c%c: size: %d\n", buf[0], buf[1], buf[2],
+    lprintf("frame: %c%c%c: size: %zu\n", buf[0], buf[1], buf[2],
             frame_header->size);
 
     return 1;
@@ -373,7 +373,7 @@ static int id3v22_parse_tag(input_plugin_t *input,
     while ((pos + ID3V22_FRAME_HEADER_SIZE) <= tag_header.size) {
       if (id3v22_parse_frame_header(input, &tag_frame_header)) {
         pos += ID3V22_FRAME_HEADER_SIZE;
-        if (tag_frame_header.id && tag_frame_header.size) {
+        if (tag_frame_header.id) {
           if ((pos + tag_frame_header.size) <= tag_header.size) {
             if (!id3v22_interp_frame(input, stream, &tag_frame_header)) {
               xprintf(stream->xine, XINE_VERBOSITY_DEBUG, 
@@ -417,7 +417,7 @@ static int id3v23_parse_frame_header(input_plugin_t *input,
     frame_header->size  = _X_BE_32(&buf[4]);
     frame_header->flags = _X_BE_16(buf + 8);
 
-    lprintf("frame: %c%c%c%c, size: %d, flags: %X\n", buf[0], buf[1], buf[2], buf[3],
+    lprintf("frame: %c%c%c%c, size: %zu, flags: %X\n", buf[0], buf[1], buf[2], buf[3],
             frame_header->size, frame_header->flags);
 
     return 1;
@@ -453,11 +453,11 @@ static int id3v23_parse_frame_ext_header(input_plugin_t *input,
       }
 
     } else {
-      lprintf("invalid ext header size: %d\n", frame_ext_header->size);
+      lprintf("invalid ext header size: %zu\n", frame_ext_header->size);
       return 0;
     }
 
-    lprintf("ext header: size: %d, flags: %X, padding_size: %d, crc: %d\n",
+    lprintf("ext header: size: %zu, flags: %X, padding_size: %d, crc: %d\n",
             frame_ext_header->size, frame_ext_header->flags,
             frame_ext_header->padding_size, frame_ext_header->crc);
     return 1;
@@ -570,7 +570,7 @@ static int id3v23_parse_tag(input_plugin_t *input,
     while ((pos + ID3V23_FRAME_HEADER_SIZE) <= tag_header.size) {
       if (id3v23_parse_frame_header(input, &tag_frame_header)) {
         pos += ID3V23_FRAME_HEADER_SIZE;
-        if (tag_frame_header.id && tag_frame_header.size) {
+        if (tag_frame_header.id) {
           if ((pos + tag_frame_header.size) <= tag_header.size) {
             if (!id3v23_interp_frame(input, stream, &tag_frame_header)) {
               xprintf(stream->xine, XINE_VERBOSITY_DEBUG, 
@@ -585,7 +585,7 @@ static int id3v23_parse_tag(input_plugin_t *input,
           pos += tag_frame_header.size;
         } else {
           /* end of frames, the rest is padding */
-	  lprintf("skipping padding %d bytes\n", tag_header.size - pos);
+	  lprintf("skipping padding %zu bytes\n", tag_header.size - pos);
 	  input->seek (input, tag_header.size - pos, SEEK_CUR);
           return 1;
         }
@@ -631,7 +631,7 @@ static int id3v24_parse_frame_header(input_plugin_t *input,
     frame_header->size  = _X_BE_32_synchsafe(&buf[4]);
     frame_header->flags = _X_BE_16(&buf[8]);
 
-    lprintf("frame: %c%c%c%c, size: %d, flags: %X\n", buf[0], buf[1], buf[2], buf[3],
+    lprintf("frame: %c%c%c%c, size: %zu, flags: %X\n", buf[0], buf[1], buf[2], buf[3],
             frame_header->size, frame_header->flags);
 
     return 1;
@@ -711,7 +711,7 @@ static int id3v24_parse_ext_header(input_plugin_t *input,
     } else {
       return 0;
     }
-    lprintf("ext header: size: %d, flags: %X, crc: %d, restrictions: %8X\n",
+    lprintf("ext header: size: %zu, flags: %X, crc: %d, restrictions: %8X\n",
             frame_ext_header->size, frame_ext_header->flags,
             frame_ext_header->crc, frame_ext_header->restrictions);
     return 1;
@@ -826,7 +826,7 @@ static int id3v24_parse_tag(input_plugin_t *input,
     while ((pos + ID3V24_FRAME_HEADER_SIZE) <= tag_header.size) {
       if (id3v24_parse_frame_header(input, &tag_frame_header)) {
         pos += ID3V24_FRAME_HEADER_SIZE;
-        if (tag_frame_header.id && tag_frame_header.size) {
+        if (tag_frame_header.id) {
           if ((pos + tag_frame_header.size) <= tag_header.size) {
             if (!id3v24_interp_frame(input, stream, &tag_frame_header)) {
               xprintf(stream->xine, XINE_VERBOSITY_DEBUG, 
