@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: video_out_dxr3.h,v 1.14 2003/01/18 17:25:41 mroi Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -30,21 +28,18 @@
 
 #include "xine_internal.h"
 #include "vo_scale.h"
+#include "dxr3_scr.h"
 #include "dxr3.h"
 
-/* values for fd_video indicating why it is closed */
-#define CLOSED_FOR_DECODER -1
-#define CLOSED_FOR_ENCODER -2
-
 /* the number of supported encoders */
-#define SUPPORTED_ENCODER_COUNT 2
+#define SUPPORTED_ENCODER_COUNT 3
 
 
 /* plugin structures */
 typedef struct encoder_data_s encoder_data_t;
 typedef struct spu_encoder_s spu_encoder_t;
 
-typedef enum { ENC_FAME, ENC_RTE } encoder_type;
+typedef enum { ENC_FAME, ENC_RTE, ENC_LAVC } encoder_type;
 
 
 struct coeff {
@@ -52,6 +47,8 @@ struct coeff {
 };
 
 typedef struct dxr3_overlay_s {
+  xine_t          *xine;
+  
   int              fd_control;
 
   int              xoffset;
@@ -64,6 +61,7 @@ typedef struct dxr3_overlay_s {
   int              screen_xres;
   int              screen_yres;
   int              screen_depth;
+  int              shrink;
 
   struct coeff     colcal_upper[3];
   struct coeff     colcal_lower[3];
@@ -76,8 +74,9 @@ typedef struct dxr3_driver_class_s {
   int                   visual_type;
   int                   instance;           /* we allow only one instance of this plugin */
   
-  char                  devname[128];
-  char                  devnum[3];
+  int                   devnum;
+  
+  dxr3_scr_t           *scr;                /* to provide dxr3 clocking */
 } dxr3_driver_class_t;
 
 typedef struct dxr3_driver_s {
@@ -85,6 +84,7 @@ typedef struct dxr3_driver_s {
   dxr3_driver_class_t *class;
 
   int                  fd_control;
+  pthread_mutex_t      video_device_lock;
   int                  fd_video;
   pthread_mutex_t      spu_device_lock;
   int                  fd_spu;              /* to access the relevant dxr3 devices */
@@ -106,13 +106,15 @@ typedef struct dxr3_driver_s {
   spu_encoder_t       *spu_enc;             /* spu encoder */
   int                  need_update;         /* the mpeg encoder needs to be updated */
   
-  int                  video_iheight;       /* input height (before adding black bars) */
-  int                  video_oheight;       /* output height (after adding black bars) */
-  int                  video_width;
-  int                  video_ratio;
+  uint32_t             video_iheight;       /* input height (before adding black bars) */
+  uint32_t             video_oheight;       /* output height (after adding black bars) */
+  uint32_t             video_width;
+  double               video_ratio;
+  int                  video_aspect;
   int                  top_bar;             /* the height of the upper black bar */
   
   vo_scale_t           scale;
+  alphablend_t         alphablend_extra_data;
 
   dxr3_overlay_t       overlay;
 #ifdef HAVE_X11
@@ -127,7 +129,7 @@ typedef struct dxr3_driver_s {
 
 typedef struct dxr3_frame_s {
   vo_frame_t       vo_frame;
-  int              oheight;
+  uint32_t         oheight;
   int              aspect, pan_scan;
   uint8_t         *mem;           /* allocated for YV12 or YUY2 buffers */
   uint8_t         *real_base[3];  /* yuv/yuy2 buffers in mem aligned on 16 */
@@ -141,6 +143,9 @@ struct encoder_data_s {
   int            (*on_display_frame)(dxr3_driver_t *, dxr3_frame_t *);
   int            (*on_unneeded)(dxr3_driver_t *);
   int            (*on_close)(dxr3_driver_t *);
+  
+  /* this is only used by the libavcodec encoder */
+  void            *handle;
 }; 
 
 struct spu_encoder_s {
@@ -152,8 +157,8 @@ struct spu_encoder_s {
   uint32_t        color[16];
   uint8_t         trans[4];
   int             map[OVL_PALETTE_SIZE];
-  uint32_t        clip_color[16];
-  uint8_t         clip_trans[4];
+  uint32_t        hili_color[16];
+  uint8_t         hili_trans[4];
   int             clip_map[OVL_PALETTE_SIZE];
 };
 
@@ -164,6 +169,7 @@ int dxr3_rte_init(dxr3_driver_t *);
 #ifdef HAVE_LIBFAME
 int dxr3_fame_init(dxr3_driver_t *);
 #endif
+int dxr3_lavc_init(dxr3_driver_t *, plugin_node_t *);
 
 /* spu encoder functions */
 spu_encoder_t *dxr3_spu_encoder_init(void);
