@@ -21,6 +21,10 @@
  * adopted from joschkas real tools.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <string.h>
 
