@@ -49,7 +49,7 @@ a52_state_t * a52_init (uint32_t mm_accel)
     a52_state_t * state;
     int i;
 
-    state = xine_xmalloc (sizeof (a52_state_t));
+    state = calloc(1, sizeof (a52_state_t));
     if (state == NULL)
 	return NULL;
 
