@@ -21,6 +21,10 @@
  * 09-12-2001 DTS passthrough inprovements (C) James Courtier-Dutton
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #ifndef __sun
 /* required for swab() */
 #define _XOPEN_SOURCE 500
@@ -489,7 +493,7 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
 
   lprintf("open_plugin\n");
 
-  this = (dts_decoder_t *) xine_xmalloc (sizeof (dts_decoder_t));
+  this = calloc(1, sizeof (dts_decoder_t));
 
   this->audio_decoder.decode_data         = dts_decode_data;
   this->audio_decoder.reset               = dts_reset;
@@ -598,7 +602,7 @@ static void *init_plugin (xine_t *xine, void *data) {
 
   lprintf("init_plugin\n");
 
-  this = (dts_class_t *) xine_xmalloc (sizeof (dts_class_t));
+  this = calloc(1, sizeof (dts_class_t));
 
   this->decoder_class.open_plugin     = open_plugin;
   this->decoder_class.get_identifier  = get_identifier;
