@@ -20,6 +20,11 @@
  * 31-8-2001 Added LPCM rate sensing.
  *   (c) 2001 James Courtier-Dutton James@superbug.demon.co.uk
  */
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #ifndef __sun
 #define _XOPEN_SOURCE 500
 #endif
@@ -240,7 +245,7 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
 
   lpcm_decoder_t *this ;
 
-  this = (lpcm_decoder_t *) xine_xmalloc (sizeof (lpcm_decoder_t));
+  this = (lpcm_decoder_t *) calloc(1, sizeof(lpcm_decoder_t));
 
   this->audio_decoder.decode_data         = lpcm_decode_data;
   this->audio_decoder.reset               = lpcm_reset;
@@ -275,7 +280,7 @@ static void *init_plugin (xine_t *xine, void *data) {
 
   lpcm_class_t *this ;
 
-  this = (lpcm_class_t *) xine_xmalloc (sizeof (lpcm_class_t));
+  this = (lpcm_class_t *) calloc(1, sizeof(lpcm_class_t));
 
   this->decoder_class.open_plugin     = open_plugin;
   this->decoder_class.get_identifier  = get_identifier;
