@@ -23,6 +23,10 @@
  * heavily based on tvtime.sf.net by Billy Biggs
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 /*
 #define LOG
 */
