@@ -23,6 +23,10 @@
  * FFT code by Steve Haehnichen, originally licensed under GPL v1
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <math.h>
 
