@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
- *
- * $Id: video_out_pgx64.c,v 1.78 2006/07/10 22:08:44 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * video_out_pgx64.c, Sun XVR100/PGX64/PGX24 output plugin for xine
  *
@@ -25,7 +23,6 @@
  *   Robin Kay <komadori [at] gekkou [dot] co [dot] uk>
  *
  * Sun XVR-100 framebuffer graciously donated by Jake Goerzen.
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -564,7 +561,7 @@ static vo_frame_t *pgx64_alloc_frame(vo_driver_t *this_gen)
   /*pgx64_driver_t *this = (pgx64_driver_t *)(void *)this_gen;*/
   pgx64_frame_t *frame;
 
-  frame = (pgx64_frame_t *) xine_xmalloc(sizeof(pgx64_frame_t));
+  frame = calloc(1, sizeof(pgx64_frame_t));
   if (!frame) {
     return NULL;
   }
@@ -1355,7 +1352,7 @@ static vo_driver_t *pgx64_init_driver(video_driver_class_t *class_gen, const voi
   struct fbgattr attr;
   long page_size;
 
-  this = (pgx64_driver_t *)xine_xmalloc(sizeof(pgx64_driver_t));
+  this = calloc(1, sizeof(pgx64_driver_t));
   if (!this) {
     return NULL;
   }
@@ -1496,7 +1493,7 @@ static void *pgx64_init_class(xine_t *xine, void *visual_gen)
 {
   pgx64_driver_class_t *class;
 
-  class = (pgx64_driver_class_t *)xine_xmalloc(sizeof(pgx64_driver_class_t));
+  class = calloc(1, sizeof(pgx64_driver_class_t));
   if (!class) {
     return NULL;
   }
