@@ -15,9 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: video_out_sdl.c,v 1.48 2006/07/10 22:08:44 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * video_out_sdl.c, Simple DirectMedia Layer
  *
@@ -140,7 +138,7 @@ static vo_frame_t *sdl_alloc_frame (vo_driver_t *this_gen) {
   /* sdl_driver_t    *this = (sdl_driver_t *) this_gen; */
   sdl_frame_t     *frame ;
 
-  frame = (sdl_frame_t *) xine_xmalloc (sizeof (sdl_frame_t));
+  frame = (sdl_frame_t *) calloc(1, sizeof(sdl_frame_t));
 
   if (!frame)
     return NULL;
@@ -362,6 +360,14 @@ static int sdl_get_property (vo_driver_t *this_gen, int property) {
       return this->sc.gui_width;
     case VO_PROP_WINDOW_HEIGHT:
       return this->sc.gui_height;
+    case VO_PROP_OUTPUT_WIDTH:
+      return this->sc.output_width;
+    case VO_PROP_OUTPUT_HEIGHT:
+      return this->sc.output_height;
+    case VO_PROP_OUTPUT_XOFFSET:
+      return this->sc.output_xoffset;
+    case VO_PROP_OUTPUT_YOFFSET:
+      return this->sc.output_yoffset;
     case VO_PROP_ASPECT_RATIO:
     return this->sc.user_ratio;
   }
@@ -463,13 +469,15 @@ static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *vi
   sdl_driver_t         *this;
 
   const SDL_VideoInfo  *vidInfo;
-#ifdef HAVE_X11
+#if defined(HAVE_X11) || defined(WIN32)
   static char           SDL_windowhack[32];
   x11_visual_t         *visual = (x11_visual_t *) visual_gen;
+#endif
+#ifdef HAVE_X11
   XWindowAttributes     window_attributes;
 #endif
   
-  this = (sdl_driver_t *) xine_xmalloc (sizeof (sdl_driver_t));
+  this = (sdl_driver_t *) calloc(1, sizeof(sdl_driver_t));
   if (!this)
     return NULL;
 
@@ -496,14 +504,18 @@ static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *vi
   _x_vo_scale_init( &this->sc, 0, 0, config);
   this->sc.frame_output_cb   = visual->frame_output_cb;
   this->sc.user_data         = visual->user_data;
-
-  /* set SDL to use our existing X11 window */
-  sprintf(SDL_windowhack,"SDL_WINDOWID=0x%x", (uint32_t) this->drawable );
-  putenv(SDL_windowhack);
 #else
   _x_vo_scale_init( &this->sc, 0, 0, config );
 #endif
 
+#if defined(HAVE_X11) || defined(WIN32)
+  /* set SDL to use our existing X11/win32 window */
+  if (visual->d){
+    sprintf(SDL_windowhack,"SDL_WINDOWID=0x%x", (uint32_t) visual->d);
+    putenv(SDL_windowhack);
+  }
+#endif
+
   if ((SDL_Init (SDL_INIT_VIDEO)) < 0) {
     xprintf (this->xine, XINE_VERBOSITY_DEBUG, "video_out_sdl: open_plugin - sdl video initialization failed.\n");
     return NULL;
@@ -590,7 +602,7 @@ static void *init_class (xine_t *xine, void *visual_gen) {
   }
   SDL_QuitSubSystem (SDL_INIT_VIDEO);
 
-  this = (sdl_class_t*) xine_xmalloc (sizeof (sdl_class_t));
+  this = (sdl_class_t*) calloc(1, sizeof(sdl_class_t));
    
   this->driver_class.open_plugin      = open_plugin;
   this->driver_class.get_identifier   = get_identifier;
