@@ -295,7 +295,7 @@ static vo_frame_t *xshm_alloc_frame (vo_driver_t *this_gen) {
   xshm_frame_t  *frame;
   xshm_driver_t *this = (xshm_driver_t *) this_gen;
 
-  frame = (xshm_frame_t *) xine_xmalloc (sizeof (xshm_frame_t));
+  frame = (xshm_frame_t *) calloc(1, sizeof(xshm_frame_t));
   if (!frame)
     return NULL;
 
@@ -1015,7 +1015,7 @@ static vo_driver_t *xshm_open_plugin(video_driver_class_t *class_gen, const void
 
   const xcb_query_extension_reply_t *query_extension_reply;
   
-  this = (xshm_driver_t *) xine_xmalloc (sizeof (xshm_driver_t));
+  this = (xshm_driver_t *) calloc(1, sizeof(xshm_driver_t));
 
   if (!this)
     return NULL;
@@ -1250,7 +1250,7 @@ static void xshm_dispose_class (video_driver_class_t *this_gen) {
 }
 
 static void *xshm_init_class (xine_t *xine, void *visual_gen) {
-  xshm_class_t	       *this = (xshm_class_t *) xine_xmalloc (sizeof (xshm_class_t));
+  xshm_class_t	       *this = (xshm_class_t *) calloc(1, sizeof(xshm_class_t));
 
   this->driver_class.open_plugin     = xshm_open_plugin;
   this->driver_class.get_identifier  = xshm_get_identifier;
