@@ -5,10 +5,6 @@
    Licence: GPL
    WARNING: THIS DRIVER IS IN BETTA STAGE
 */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -16,6 +12,7 @@
 #include <math.h>
 #include <inttypes.h>
 #include <fcntl.h>
+#include <limits.h>
 #include <sys/mman.h> /* for m(un)lock */
 #ifdef HAVE_MALLOC_H
 #include <malloc.h>
@@ -24,17 +21,18 @@
 #endif
 #endif
 
-
 #include "vidix.h"
 #include "fourcc.h"
 #include "libdha.h"
 #include "pci_ids.h"
 #include "pci_names.h"
+#include "bswap.h"
 
 #include "mach64.h"
 
 #define UNUSED(x) ((void)(x)) /**< Removes warning about unused arguments */
 
+#define VIDIX_STATIC mach64_
 
 #ifdef MACH64_ENABLE_BM
 
@@ -174,8 +172,11 @@ static video_registers_t vregs[] =
 
 #define INREG8(addr)		GETREG(uint8_t,(uint32_t)mach64_mmio_base,((addr)^0x100)<<2)
 #define OUTREG8(addr,val)	SETREG(uint8_t,(uint32_t)mach64_mmio_base,((addr)^0x100)<<2,val)
-#define INREG(addr)		GETREG(uint32_t,(uint32_t)mach64_mmio_base,((addr)^0x100)<<2)
-#define OUTREG(addr,val)	SETREG(uint32_t,(uint32_t)mach64_mmio_base,((addr)^0x100)<<2,val)
+static inline uint32_t INREG (uint32_t addr) {
+    uint32_t tmp = GETREG(uint32_t,(uint32_t)mach64_mmio_base,((addr)^0x100)<<2);
+    return le2me_32(tmp);
+}
+#define OUTREG(addr,val)	SETREG(uint32_t,(uint32_t)mach64_mmio_base,((addr)^0x100)<<2,le2me_32(val))
 
 #define OUTREGP(addr,val,mask)  					\
 	do {								\
@@ -227,6 +228,7 @@ static void mach64_engine_reset( void )
 {
   /* Kill off bus mastering with extreme predjudice... */
   OUTREG(BUS_CNTL, INREG(BUS_CNTL) | BUS_MASTER_DIS);
+  OUTREG(CRTC_INT_CNTL,INREG(CRTC_INT_CNTL)&~(CRTC_BUSMASTER_EOL_INT|CRTC_BUSMASTER_EOL_INT_EN));
   /* Reset engine -- This is accomplished by setting bit 8 of the GEN_TEST_CNTL
    register high, then low (per the documentation, it's on high to low transition
    that the GUI engine gets reset...) */
@@ -385,70 +387,79 @@ static void mach64_vid_dump_regs( void )
   for(i=0;i<sizeof(vregs)/sizeof(video_registers_t);i++)
   {
 	mach64_wait_for_idle();
+	mach64_fifo_wait(2);
 	printf("[mach64] %s = %08X\n",vregs[i].sname,INREG(vregs[i].name));
   }
   printf("[mach64] *** End of OV0 registers dump ***\n");
 }
 
 
-unsigned int vixGetVersion(void)
+unsigned int VIDIX_NAME(vixGetVersion)(void)
 {
     return(VIDIX_VERSION);
 }
 
-static unsigned short ati_card_ids[] = 
+typedef struct ati_chip_id_s
 {
- DEVICE_ATI_215CT_MACH64_CT,
- DEVICE_ATI_210888CX_MACH64_CX,
- DEVICE_ATI_210888ET_MACH64_ET,
- DEVICE_ATI_MACH64_VT,
- DEVICE_ATI_210888GX_MACH64_GX,
- DEVICE_ATI_264LT_MACH64_LT,
- DEVICE_ATI_264VT_MACH64_VT,
- DEVICE_ATI_264VT3_MACH64_VT3,
- DEVICE_ATI_264VT4_MACH64_VT4,
+    unsigned short	id;
+    unsigned short	is_agp;
+}ati_chip_id_t;
+
+static ati_chip_id_t ati_card_ids[] = 
+{
+ { DEVICE_ATI_215CT_MACH64_CT, 0 },
+ { DEVICE_ATI_210888CX_MACH64_CX, 0 },
+ { DEVICE_ATI_210888ET_MACH64_ET, 0 },
+ { DEVICE_ATI_MACH64_VT, 0 },
+ { DEVICE_ATI_210888GX_MACH64_GX, 0 },
+ { DEVICE_ATI_264LT_MACH64_LT, 0 },
+ { DEVICE_ATI_264VT_MACH64_VT, 0 },
+ { DEVICE_ATI_264VT3_MACH64_VT3, 0 },
+ { DEVICE_ATI_264VT4_MACH64_VT4, 0 },
  /**/
- DEVICE_ATI_3D_RAGE_PRO,
- DEVICE_ATI_3D_RAGE_PRO2,
- DEVICE_ATI_3D_RAGE_PRO3,
- DEVICE_ATI_3D_RAGE_PRO4,
- DEVICE_ATI_RAGE_XC,
- DEVICE_ATI_RAGE_XL_AGP,
- DEVICE_ATI_RAGE_XC_AGP,
- DEVICE_ATI_RAGE_XL,
- DEVICE_ATI_3D_RAGE_PRO5,
- DEVICE_ATI_3D_RAGE_PRO6,
- DEVICE_ATI_RAGE_XL2,
- DEVICE_ATI_RAGE_XC2,
- DEVICE_ATI_3D_RAGE_I_II,
- DEVICE_ATI_3D_RAGE_II,
- DEVICE_ATI_3D_RAGE_IIC,
- DEVICE_ATI_3D_RAGE_IIC2,
- DEVICE_ATI_3D_RAGE_IIC3,
- DEVICE_ATI_3D_RAGE_IIC4,
- DEVICE_ATI_3D_RAGE_LT,
- DEVICE_ATI_3D_RAGE_LT2,
- DEVICE_ATI_3D_RAGE_LT_G,
- DEVICE_ATI_3D_RAGE_LT3,
- DEVICE_ATI_RAGE_MOBILITY_P_M,
- DEVICE_ATI_RAGE_MOBILITY_L,
- DEVICE_ATI_3D_RAGE_LT4,
- DEVICE_ATI_3D_RAGE_LT5,
- DEVICE_ATI_RAGE_MOBILITY_P_M2,
- DEVICE_ATI_RAGE_MOBILITY_L2
+ { DEVICE_ATI_3D_RAGE_PRO, 1 },
+ { DEVICE_ATI_3D_RAGE_PRO2, 1 },
+ { DEVICE_ATI_3D_RAGE_PRO3, 0 },
+ { DEVICE_ATI_3D_RAGE_PRO4, 0 },
+ { DEVICE_ATI_RAGE_XC, 0 },
+ { DEVICE_ATI_RAGE_XL_AGP, 1 },
+ { DEVICE_ATI_RAGE_XC_AGP, 1 },
+ { DEVICE_ATI_RAGE_XL, 0 },
+ { DEVICE_ATI_3D_RAGE_PRO5, 0 },
+ { DEVICE_ATI_3D_RAGE_PRO6, 0 },
+ { DEVICE_ATI_RAGE_XL2, 0 },
+ { DEVICE_ATI_RAGE_XC2, 0 },
+ { DEVICE_ATI_3D_RAGE_I_II, 0 },
+ { DEVICE_ATI_3D_RAGE_II, 0 },
+ { DEVICE_ATI_3D_RAGE_IIC, 1 },
+ { DEVICE_ATI_3D_RAGE_IIC2, 0 },
+ { DEVICE_ATI_3D_RAGE_IIC3, 0 },
+ { DEVICE_ATI_3D_RAGE_IIC4, 1 },
+ { DEVICE_ATI_3D_RAGE_LT, 1 },
+ { DEVICE_ATI_3D_RAGE_LT2, 1 },
+ { DEVICE_ATI_3D_RAGE_LT_G, 0 },
+ { DEVICE_ATI_3D_RAGE_LT3, 0 },
+ { DEVICE_ATI_RAGE_MOBILITY_P_M, 1 },
+ { DEVICE_ATI_RAGE_MOBILITY_L, 1 },
+ { DEVICE_ATI_3D_RAGE_LT4, 0 },
+ { DEVICE_ATI_3D_RAGE_LT5, 0 },
+ { DEVICE_ATI_RAGE_MOBILITY_P_M2, 0 },
+ { DEVICE_ATI_RAGE_MOBILITY_L2, 0 }
 };
 
+static int is_agp;
+
 static int find_chip(unsigned chip_id)
 {
   unsigned i;
-  for(i = 0;i < sizeof(ati_card_ids)/sizeof(unsigned short);i++)
+  for(i = 0;i < sizeof(ati_card_ids)/sizeof(ati_chip_id_t);i++)
   {
-    if(chip_id == ati_card_ids[i]) return i;
+    if(chip_id == ati_card_ids[i].id) return i;
   }
   return -1;
 }
 
-int vixProbe(int verbose,int force)
+int VIDIX_NAME(vixProbe)(int verbose,int force)
 {
   pciinfo_t lst[MAX_PCI_DEVICES];
   unsigned i,num_pci;
@@ -480,6 +491,7 @@ int vixProbe(int verbose,int force)
 	    if(idx == -1)
 		printf("[mach64] Assuming it as Mach64\n");
 	}
+	if(idx != -1) is_agp = ati_card_ids[idx].is_agp;
 	mach64_cap.device_id = lst[i].device;
 	err = 0;
 	memcpy(&pci_info,&lst[i],sizeof(pciinfo_t));
@@ -502,17 +514,82 @@ static void reset_regs( void )
   }
 }
 
-int vixInit(void)
+typedef struct saved_regs_s
+{
+    uint32_t overlay_video_key_clr;
+    uint32_t overlay_video_key_msk;
+    uint32_t overlay_graphics_key_clr;
+    uint32_t overlay_graphics_key_msk;
+    uint32_t overlay_key_cntl;
+    uint32_t bus_cntl;
+}saved_regs_t;
+static saved_regs_t savreg;
+
+static void save_regs( void )
+{
+    mach64_fifo_wait(6);
+    savreg.overlay_video_key_clr	= INREG(OVERLAY_VIDEO_KEY_CLR);
+    savreg.overlay_video_key_msk	= INREG(OVERLAY_VIDEO_KEY_MSK);
+    savreg.overlay_graphics_key_clr	= INREG(OVERLAY_GRAPHICS_KEY_CLR);
+    savreg.overlay_graphics_key_msk	= INREG(OVERLAY_GRAPHICS_KEY_MSK);
+    savreg.overlay_key_cntl		= INREG(OVERLAY_KEY_CNTL);
+    savreg.bus_cntl			= INREG(BUS_CNTL);
+}
+
+static void restore_regs( void )
+{
+    mach64_fifo_wait(6);
+    OUTREG(OVERLAY_VIDEO_KEY_CLR,savreg.overlay_video_key_clr);
+    OUTREG(OVERLAY_VIDEO_KEY_MSK,savreg.overlay_video_key_msk);
+    OUTREG(OVERLAY_GRAPHICS_KEY_CLR,savreg.overlay_graphics_key_clr);
+    OUTREG(OVERLAY_GRAPHICS_KEY_MSK,savreg.overlay_graphics_key_msk);
+    OUTREG(OVERLAY_KEY_CNTL,savreg.overlay_key_cntl);
+    OUTREG(BUS_CNTL,savreg.bus_cntl|BUS_MASTER_DIS);
+}
+
+static int forced_irq=INT_MAX;
+
+#ifdef MACH64_ENABLE_BM
+static int can_use_irq=0;
+static int irq_installed=0;
+
+static void init_irq(void)
+{
+	irq_installed=1;
+	if(forced_irq != INT_MAX) pci_info.irq=forced_irq;
+	if(hwirq_install(pci_info.bus,pci_info.card,pci_info.func,
+			 2,CRTC_INT_CNTL,CRTC_BUSMASTER_EOL_INT) == 0)
+	{
+	    can_use_irq=1;
+	    if(__verbose) printf("[mach64] Will use %u irq line\n",pci_info.irq);
+	}
+	else 
+	    if(__verbose) printf("[mach64] Can't initialize irq handling: %s\n"
+				 "[mach64]irq_param: line=%u pin=%u gnt=%u lat=%u\n"
+				 ,strerror(errno)
+				 ,pci_info.irq,pci_info.ipin,pci_info.gnt,pci_info.lat);
+}
+#endif
+
+int VIDIX_NAME(vixInit)(const char *args)
 {
   int err;
+#ifdef MACH64_ENABLE_BM
   unsigned i;
+#endif
   if(!probed)
   {
     printf("[mach64] Driver was not probed but is being initializing\n");
     return EINTR;
   }
-  if(__verbose>0) printf("[mach64] version %s\n", VERSION);
-  
+  if(__verbose>0) printf("[mach64] version %d args='%s'\n", VIDIX_VERSION,args);
+  if(args)
+  if(strncmp(args,"irq=",4) == 0) 
+  {
+    forced_irq=atoi(&args[4]);
+    if(__verbose>0) printf("[mach64] forcing IRQ to %u\n",forced_irq);     
+  }
+
   if((mach64_mmio_base = map_phys_mem(pci_info.base2,0x4000))==(void *)-1) return ENOMEM;
   mach64_wait_for_idle();
   mach64_ram_size = INREG(MEM_CNTL) & CTL_MEM_SIZEB;
@@ -526,6 +603,7 @@ int vixInit(void)
   err = mtrr_set_type(pci_info.base0,mach64_ram_size,MTRR_TYPE_WRCOMB);
   if(!err) printf("[mach64] Set write-combining type of video memory\n");
   
+  save_regs();
   /* check if planar formats are supported */
   supports_planar=0;
   mach64_wait_for_idle();
@@ -567,13 +645,15 @@ int vixInit(void)
   mach64_vid_make_default();
   if(__verbose > VERBOSE_LEVEL) mach64_vid_dump_regs();
 #ifdef MACH64_ENABLE_BM
+  if(!(INREG(BUS_CNTL) & BUS_MASTER_DIS))
+		OUTREG(BUS_CNTL,INREG(BUS_CNTL)|BUS_MSTR_RESET);
   if(bm_open() == 0)
   {
 	mach64_cap.flags |= FLAG_DMA | FLAG_EQ_DMA;
 	if((dma_phys_addrs = malloc(mach64_ram_size*sizeof(unsigned long)/4096)) == 0)
 	{
 	    out_mem:
-	    printf("[mach64] Can't allocate temopary buffer for DMA\n");
+	    printf("[mach64] Can't allocate temporary buffer for DMA\n");
 	    mach64_cap.flags &= ~FLAG_DMA & ~FLAG_EQ_DMA;
 	    return 0;
 	}
@@ -585,6 +665,15 @@ int vixInit(void)
 	for(i=0;i<64;i++)
 	    if((mach64_dma_desc_base[i] = memalign(4096,mach64_ram_size*sizeof(bm_list_descriptor)/4096)) == 0)
 		goto out_mem;
+#if 0
+	if(!is_agp)
+	{
+	    long tst;
+	    if(pci_config_read(pci_info.bus,pci_info.card,pci_info.func,4,4,&pci_command) == 0)
+		pci_config_write(pci_info.bus,pci_info.card,pci_info.func,4,4,pci_command|0x14);
+	    pci_config_read(pci_info.bus,pci_info.card,pci_info.func,4,4,&tst);
+	}
+#endif
   }
   else
     if(__verbose) printf("[mach64] Can't initialize busmastering: %s\n",strerror(errno));
@@ -592,13 +681,20 @@ int vixInit(void)
   return 0;
 }
 
-void vixDestroy(void)
+void VIDIX_NAME(vixDestroy)(void)
 {
+#ifdef MACH64_ENABLE_BM
   unsigned i;
+#endif
+  restore_regs();
+#ifdef MACH64_ENABLE_BM
+  mach64_engine_reset();
+#endif
   unmap_phys_mem(mach64_mem_base,mach64_ram_size);
   unmap_phys_mem(mach64_mmio_base,0x4000);
 #ifdef MACH64_ENABLE_BM
   bm_close();
+  if(can_use_irq && irq_installed) hwirq_uninstall(pci_info.bus,pci_info.card,pci_info.func);
   if(dma_phys_addrs) free(dma_phys_addrs);
   for(i=0;i<64;i++) 
   {
@@ -607,7 +703,7 @@ void vixDestroy(void)
 #endif
 }
 
-int vixGetCapability(vidix_capability_t *to)
+int VIDIX_NAME(vixGetCapability)(vidix_capability_t *to)
 {
     memcpy(to, &mach64_cap, sizeof(vidix_capability_t));
     return 0;
@@ -745,9 +841,8 @@ static void mach64_vid_display_video( void )
 	
        As for me - I would prefer to limit movie's width with 360 but it provides only
        half of picture but with perfect quality. (NK) */
-    mach64_fifo_wait(4);
+    mach64_fifo_wait(10);
     OUTREG(OVERLAY_SCALE_CNTL, sc);
-
     mach64_wait_for_idle();
 
     switch(besr.fourcc)
@@ -886,6 +981,8 @@ for(i=0; i<32; i++){
 	  tmp = config->offset.u;
 	  config->offset.u = config->offset.v;
 	  config->offset.v = tmp;
+	  src_offset_u=config->offset.u;
+	  src_offset_v=config->offset.v;
 	}
     }
     else if(besr.fourcc == IMGFMT_YVU9)
@@ -931,7 +1028,6 @@ for(i=0; i<32; i++){
     if(mach64_is_interlace()) y_pos/=2;
     besr.y_x_end = y_pos | ((config->dest.x + dest_w) << 16);
     besr.height_width = ((src_w - left)<<16) | (src_h - top);
-
     return 0;
 }
 
@@ -955,7 +1051,7 @@ static int is_supported_fourcc(uint32_t fourcc)
     }
 }
 
-int vixQueryFourcc(vidix_fourcc_t *to)
+int VIDIX_NAME(vixQueryFourcc)(vidix_fourcc_t *to)
 {
     if(is_supported_fourcc(to->fourcc))
     {
@@ -971,7 +1067,7 @@ int vixQueryFourcc(vidix_fourcc_t *to)
     return ENOSYS;
 }
 
-int vixConfigPlayback(vidix_playback_t *info)
+int VIDIX_NAME(vixConfigPlayback)(vidix_playback_t *info)
 {
   unsigned rgb_size,nfr;
   uint32_t mach64_video_size;
@@ -1011,7 +1107,7 @@ int vixConfigPlayback(vidix_playback_t *info)
   return 0;
 }
 
-int vixPlaybackOn(void)
+int VIDIX_NAME(vixPlaybackOn)(void)
 {
   int err;
   unsigned dw,dh;
@@ -1029,13 +1125,13 @@ int vixPlaybackOn(void)
   return err;
 }
 
-int vixPlaybackOff(void)
+int VIDIX_NAME(vixPlaybackOff)(void)
 {
   mach64_vid_stop_video();
   return 0;
 }
 
-int vixPlaybackFrameSelect(unsigned int frame)
+int VIDIX_NAME(vixPlaybackFrameSelect)(unsigned int frame)
 {
     uint32_t off[6];
     int i;
@@ -1080,14 +1176,14 @@ vidix_video_eq_t equal =
  ,
  0, 0, 0, 0, 0, 0, 0, 0 };
 
-int 	vixPlaybackGetEq( vidix_video_eq_t * eq)
+int 	VIDIX_NAME(vixPlaybackGetEq)( vidix_video_eq_t * eq)
 {
   memcpy(eq,&equal,sizeof(vidix_video_eq_t));
   if(!supports_colour_adj) eq->cap = VEQ_CAP_BRIGHTNESS;
   return 0;
 }
 
-int 	vixPlaybackSetEq( const vidix_video_eq_t * eq)
+int 	VIDIX_NAME(vixPlaybackSetEq)( const vidix_video_eq_t * eq)
 {
   int br,sat;
     if(eq->cap & VEQ_CAP_BRIGHTNESS) equal.brightness = eq->brightness;
@@ -1126,13 +1222,13 @@ int 	vixPlaybackSetEq( const vidix_video_eq_t * eq)
   return 0;
 }
 
-int vixGetGrKeys(vidix_grkey_t *grkey)
+int VIDIX_NAME(vixGetGrKeys)(vidix_grkey_t *grkey)
 {
     memcpy(grkey, &mach64_grkey, sizeof(vidix_grkey_t));
     return(0);
 }
 
-int vixSetGrKeys(const vidix_grkey_t *grkey)
+int VIDIX_NAME(vixSetGrKeys)(const vidix_grkey_t *grkey)
 {
     memcpy(&mach64_grkey, grkey, sizeof(vidix_grkey_t));
 
@@ -1213,6 +1309,9 @@ static int mach64_setup_frame( vidix_dma_t * dmai )
 	dmai->internal[dmai->idx] = mach64_dma_desc_base[dmai->idx];
 	dest_ptr = dmai->dest_offset;
 	count = dmai->size;
+#if 0
+printf("MACH64_DMA_REQUEST va=%X size=%X\n",dmai->src,dmai->size);
+#endif
 	for(i=0;i<n;i++)
 	{
 	    list[i].framebuf_offset = mach64_overlay_offset + dest_ptr; /* offset within of video memory */
@@ -1220,42 +1319,72 @@ static int mach64_setup_frame( vidix_dma_t * dmai )
 	    list[i].command = (count > 4096 ? 4096 : (count | DMA_GUI_COMMAND__EOL));
 	    list[i].reserved = 0;
 #if 0
-printf("MACH64_DMA_TABLE[%i] %X %X %X %X\n",i,list[i].framebuf_offset,list[i].sys_addr,list[i].command,list[i].reserved);
+printf("MACH64_DMA_TABLE[%i] fboff=%X pa=%X cmd=%X rsrvd=%X\n",i,list[i].framebuf_offset,list[i].sys_addr,list[i].command,list[i].reserved);
 #endif
 	    dest_ptr += 4096;
 	    count -= 4096;
 	}
+	cpu_flush(list,4096);
     }
     return 0;
 }
 
-static int mach64_transfer_frame( unsigned long ba_dma_desc )
+static int mach64_transfer_frame( unsigned long ba_dma_desc,int sync_mode )
 {
+    uint32_t crtc_int;
     mach64_wait_for_idle();
-    OUTREG(BUS_CNTL,(INREG(BUS_CNTL)|BUS_MSTR_RESET));
-    OUTREG(CRTC_INT_CNTL,INREG(CRTC_INT_CNTL)|CRTC_BUSMASTER_EOL_INT|CRTC_BUSMASTER_EOL_INT_EN);
-    OUTREG(BUS_CNTL,(INREG(BUS_CNTL)|BUS_EXT_REG_EN|BUS_READ_BURST|BUS_PCI_READ_RETRY_EN) &(~BUS_MASTER_DIS));
+    mach64_fifo_wait(4);
+    OUTREG(BUS_CNTL,(INREG(BUS_CNTL)|BUS_EXT_REG_EN)&(~BUS_MASTER_DIS));
+    crtc_int = INREG(CRTC_INT_CNTL);
+    if(sync_mode && can_use_irq) OUTREG(CRTC_INT_CNTL,crtc_int|CRTC_BUSMASTER_EOL_INT|CRTC_BUSMASTER_EOL_INT_EN);
+    else			 OUTREG(CRTC_INT_CNTL,crtc_int|CRTC_BUSMASTER_EOL_INT);
     OUTREG(BM_SYSTEM_TABLE,ba_dma_desc|SYSTEM_TRIGGER_SYSTEM_TO_VIDEO);
     if(__verbose > VERBOSE_LEVEL) mach64_vid_dump_regs();    
+#if 0
+    mach64_fifo_wait(4);
+    mach64_fifo_wait(16);
+    printf("MACH64_DMA_DBG: bm_fb_off=%08X bm_sysmem_addr=%08X bm_cmd=%08X bm_status=%08X bm_agp_base=%08X bm_agp_cntl=%08X\n",
+	    INREG(BM_FRAME_BUF_OFFSET),
+	    INREG(BM_SYSTEM_MEM_ADDR),
+	    INREG(BM_COMMAND),
+	    INREG(BM_STATUS),
+	    INREG(AGP_BASE),
+	    INREG(AGP_CNTL));
+#endif
     return 0;
 }
 
-
-int vixPlaybackCopyFrame( vidix_dma_t * dmai )
+int VIDIX_NAME(vixQueryDMAStatus)( void )
 {
-    int retval;
+    int bm_off;
+    unsigned crtc_int_cntl;
+    mach64_wait_for_idle();
+    mach64_fifo_wait(2);
+    crtc_int_cntl = INREG(CRTC_INT_CNTL);
+    bm_off = crtc_int_cntl & CRTC_BUSMASTER_EOL_INT;
+//    if(bm_off) OUTREG(CRTC_INT_CNTL,crtc_int_cntl | CRTC_BUSMASTER_EOL_INT);
+    return bm_off?0:1;
+}
+
+int VIDIX_NAME(vixPlaybackCopyFrame)( vidix_dma_t * dmai )
+{
+    int retval,sync_mode;
     if(!(dmai->flags & BM_DMA_FIXED_BUFFS)) if(bm_lock_mem(dmai->src,dmai->size) != 0) return errno;
+    sync_mode = (dmai->flags & BM_DMA_SYNC) == BM_DMA_SYNC;
+    if(sync_mode)
+    {
+	if(!irq_installed) init_irq();
+	/* burn CPU instead of PCI bus here */
+	while(vixQueryDMAStatus()!=0){
+	    if(can_use_irq)	hwirq_wait(pci_info.irq);
+	    else		usleep(0); /* ugly but may help */
+	}
+    }
+    mach64_engine_reset();
     retval = mach64_setup_frame(dmai);
     VIRT_TO_CARD(mach64_dma_desc_base[dmai->idx],1,&bus_addr_dma_desc);
-    if(retval == 0) retval = mach64_transfer_frame(bus_addr_dma_desc);
+    if(retval == 0) retval = mach64_transfer_frame(bus_addr_dma_desc,sync_mode);
     if(!(dmai->flags & BM_DMA_FIXED_BUFFS)) bm_unlock_mem(dmai->src,dmai->size);
     return retval;
 }
-
-int	vixQueryDMAStatus( void )
-{
-    int bm_off;
-    bm_off = INREG(CRTC_INT_CNTL) & CRTC_BUSMASTER_EOL_INT;
-    return bm_off?0:1;
-}
 #endif
