@@ -17,12 +17,16 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
  *
  *------------------------------------------------------------
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 /*
 #define LOG_BLEND_YUV
 #define LOG_BLEND_RGB16
@@ -1110,12 +1114,12 @@ static uint8_t *(*blend_yuv_grow_extra_data(alphablend_t *extra_data, int osd_wi
     uint8_t *data[ 3 ][ 2 ];
   } *header = (struct header_s *)extra_data->buffer;
   
-  int needed_buffer_size = sizeof (*header) + osd_width * sizeof (uint8_t[ 3 ][ 2 ]);
+  size_t needed_buffer_size = sizeof (*header) + osd_width * sizeof (uint8_t[ 3 ][ 2 ]);
   
   if (extra_data->buffer_size < needed_buffer_size) {
     
     free(extra_data->buffer);
-    header = xine_xmalloc(needed_buffer_size);
+    header = calloc(1, needed_buffer_size);
     if (!header) {
       extra_data->buffer_size = 0;
       return 0;
@@ -1552,12 +1556,12 @@ static uint8_t *(*blend_yuy2_grow_extra_data(alphablend_t *extra_data, int osd_w
     uint8_t *data[ 3 ];
   } *header = (struct header_s *)extra_data->buffer;
   
-  int needed_buffer_size = sizeof (*header) + osd_width * sizeof (uint8_t[ 3 ]);
+  size_t needed_buffer_size = sizeof (*header) + osd_width * sizeof (uint8_t[ 3 ]);
   
   if (extra_data->buffer_size < needed_buffer_size) {
     
     free(extra_data->buffer);
-    header = xine_xmalloc(needed_buffer_size);
+    header = calloc(1, needed_buffer_size);
     if (!header) {
       extra_data->buffer_size = 0;
       return 0;
@@ -1959,7 +1963,7 @@ static int xx44_paletteIndex(xx44_palette_t *p, int color, uint32_t clut)
   }
 
   if (p->max_used == p->size -1) {
-    printf("video_out: Warning! Out of xx44 palette colors!\n");
+    printf("video_out: Warning! Out of xx44 palette colours!\n");
     return 1;
   }
   p->cluts[p->max_used] = clut;
