@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2005 the xine project
+ * Copyright (C) 2000-2008 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -659,6 +659,8 @@ static const video_db_t video_db[] = {
     ME_FOURCC('a','v','c','1'),
     ME_FOURCC('h','2','6','4'),
     ME_FOURCC('H','2','6','4'),
+    ME_FOURCC('x','2','6','4'),
+    ME_FOURCC('X','2','6','4'),
     0
   },
   BUF_VIDEO_H264,
@@ -1198,7 +1200,7 @@ static uint32_t cached_buf_type=0;
   return 0;
 }
 
-char * _x_buf_video_name( uint32_t buf_type ) {
+const char * _x_buf_video_name( uint32_t buf_type ) {
 int i;
   
   buf_type &= 0xffff0000;
@@ -1232,7 +1234,7 @@ static uint32_t cached_buf_type=0;
   return 0;
 }
 
-char * _x_buf_audio_name( uint32_t buf_type ) {
+const char * _x_buf_audio_name( uint32_t buf_type ) {
 int i;
   
   buf_type &= 0xffff0000;
