@@ -15,12 +15,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: configfile.h,v 1.42 2007/02/22 15:49:16 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * config file management
- *
  */
 
 #ifndef HAVE_CONFIGFILE_H
@@ -203,7 +200,7 @@ struct config_values_s {
 /*
  * allocate and init a new xine config object
  */
-config_values_t *_x_config_init (void) XINE_PROTECTED;
+config_values_t *_x_config_init (void) XINE_MALLOC XINE_PROTECTED;
 
 /*
  * interpret stream_setup part of mrls for config value changes
