@@ -15,14 +15,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * Buffered Input Plugin (request optimizer).
  *
  * The goal of this input plugin is to reduce 
  * the number of calls to the real input plugin.
- *
- * $Id: input_cache.c,v 1.14 2007/02/20 00:34:57 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -36,6 +34,7 @@
 */
 
 #include "xine_internal.h"
+#include <assert.h>
 
 #define DEFAULT_BUFFER_SIZE 1024
 
@@ -192,6 +191,7 @@ static buf_element_t *cache_plugin_read_block(input_plugin_t *this_gen, fifo_buf
     if (buf) {
       buf->type = BUF_DEMUX_BLOCK;
    
+      assert(todo <= buf->max_size);
       read_len = cache_plugin_read (this_gen, buf->content, todo);
       buf->size = read_len;
     }
@@ -356,7 +356,7 @@ input_plugin_t *_x_cache_plugin_get_instance (xine_stream_t *stream, int readahe
 
   lprintf("mrl: %s\n", main_plugin->get_mrl(main_plugin));
 
-  this = (cache_input_plugin_t *)xine_xmalloc(sizeof(cache_input_plugin_t));
+  this = calloc(1, sizeof(cache_input_plugin_t));
   if (!this)
     return NULL;
   
@@ -386,7 +386,7 @@ input_plugin_t *_x_cache_plugin_get_instance (xine_stream_t *stream, int readahe
     this->buf_size = DEFAULT_BUFFER_SIZE;
   }
 
-  this->buf = (char *)xine_xmalloc(this->buf_size);
+  this->buf = calloc(1, this->buf_size);
   if (!this->buf) {
     free (this);
     return NULL;
