@@ -22,6 +22,10 @@
  * Takes into account aspect ratio correction and zooming.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <string.h>
 #include <math.h>
