@@ -15,10 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: array.c,v 1.4 2007/02/03 16:31:55 dsalt Exp $
- *
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -59,7 +56,7 @@ xine_array_t *xine_array_new(size_t initial_size) {
   if (initial_size < MIN_CHUNK_SIZE)
     initial_size = MIN_CHUNK_SIZE;
 
-  new_array->chunk = (void**)malloc(sizeof(void*) * initial_size);
+  new_array->chunk = (void**)calloc(initial_size, sizeof(void*));
   if (!new_array->chunk) {
     free(new_array);
     return NULL;
