@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: xine_buffer.c,v 1.11 2006/06/23 18:24:22 dsalt Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  *
  * generic dynamic buffer functions. The goals
@@ -49,6 +47,10 @@
  *   (eg during reallocation). The user must respect that.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -119,7 +121,7 @@ typedef struct {
  */
 void *xine_buffer_init(int chunk_size) {
   
-  uint8_t *data=xine_xmalloc(chunk_size+XINE_BUFFER_HEADER_SIZE);
+  uint8_t *data=calloc(1, chunk_size+XINE_BUFFER_HEADER_SIZE);
   xine_buffer_header_t *header=(xine_buffer_header_t*)data;
 
   header->size=chunk_size;
@@ -164,7 +166,7 @@ void *xine_buffer_dup(const void *buf) {
   CHECK_MAGIC(buf);
 #endif
 
-new=xine_xmalloc(GET_HEADER(buf)->size+XINE_BUFFER_HEADER_SIZE);
+  new = malloc(GET_HEADER(buf)->size+XINE_BUFFER_HEADER_SIZE);
 
   xine_fast_memcpy(new, ((uint8_t*)buf)-XINE_BUFFER_HEADER_SIZE, 
       GET_HEADER(buf)->size+XINE_BUFFER_HEADER_SIZE);
